package digitalocean

import (
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"

	"github.com/digitalocean/terraform-provider-digitalocean/internal/mutexkv"
)

// Global MutexKV
var mutexKV = mutexkv.NewMutexKV()

// Provider returns a schema.Provider for DigitalOcean.
func Provider() *schema.Provider {
	p := &schema.Provider{
		Schema: map[string]*schema.Schema{
			"token": {
				Type:     schema.TypeString,
				Optional: true,
				DefaultFunc: schema.MultiEnvDefaultFunc([]string{
					"DIGITALOCEAN_TOKEN",
					"DIGITALOCEAN_ACCESS_TOKEN",
				}, nil),
				Description: "The token key for API operations.",
			},
			"api_endpoint": {
				Type:        schema.TypeString,
				Required:    true,
				DefaultFunc: schema.EnvDefaultFunc("DIGITALOCEAN_API_URL", "https://api.digitalocean.com"),
				Description: "The URL to use for the DigitalOcean API.",
			},
			"spaces_endpoint": {
				Type:        schema.TypeString,
				Optional:    true,
				DefaultFunc: schema.EnvDefaultFunc("SPACES_ENDPOINT_URL", "https://{{.Region}}.digitaloceanspaces.com"),
				Description: "The URL to use for the DigitalOcean Spaces API.",
			},
			"spaces_access_id": {
				Type:        schema.TypeString,
				Optional:    true,
				DefaultFunc: schema.EnvDefaultFunc("SPACES_ACCESS_KEY_ID", nil),
				Description: "The access key ID for Spaces API operations.",
			},
			"spaces_secret_key": {
				Type:        schema.TypeString,
				Optional:    true,
				DefaultFunc: schema.EnvDefaultFunc("SPACES_SECRET_ACCESS_KEY", nil),
				Description: "The secret access key for Spaces API operations.",
			},
		},
		DataSourcesMap: map[string]*schema.Resource{
			"digitalocean_droplet":               dataSourceDigitalOceanDroplet(),
			"digitalocean_droplets":              dataSourceDigitalOceanDroplets(),
			"digitalocean_image":                 dataSourceDigitalOceanImage(),
			"digitalocean_images":                dataSourceDigitalOceanImages(),
			"digitalocean_record":                dataSourceDigitalOceanRecord(),
			"digitalocean_records":               dataSourceDigitalOceanRecords(),
			"digitalocean_region":                dataSourceDigitalOceanRegion(),
			"digitalocean_regions":               dataSourceDigitalOceanRegions(),
			"digitalocean_ssh_key":               dataSourceDigitalOceanSSHKey(),
			"digitalocean_ssh_keys":              dataSourceDigitalOceanSSHKeys(),
		},

		ResourcesMap: map[string]*schema.Resource{
			"digitalocean_droplet":                               resourceDigitalOceanDroplet(),
			"digitalocean_record":                                resourceDigitalOceanRecord(),
			"digitalocean_ssh_key":                               resourceDigitalOceanSSHKey(),
		},
	}

	p.ConfigureFunc = func(d *schema.ResourceData) (interface{}, error) {
		terraformVersion := p.TerraformVersion
		if terraformVersion == "" {
			// Terraform 0.12 introduced this field to the protocol
			// We can therefore assume that if it's missing it's 0.10 or 0.11
			terraformVersion = "0.11+compatible"
		}
		return providerConfigure(d, terraformVersion)
	}

	return p
}

func providerConfigure(d *schema.ResourceData, terraformVersion string) (interface{}, error) {
	config := Config{
		Token:            d.Get("token").(string),
		APIEndpoint:      d.Get("api_endpoint").(string),
		AccessID:         d.Get("spaces_access_id").(string),
		SecretKey:        d.Get("spaces_secret_key").(string),
		TerraformVersion: terraformVersion,
	}

	if endpoint, ok := d.GetOk("spaces_endpoint"); ok {
		config.SpacesAPIEndpoint = endpoint.(string)
	}

	return config.Client()
}
