package digitalocean

import (
	"context"
	"log"
	"strings"

	"github.com/digitalocean/godo"
	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/validation"
)

func resourceDigitalOceanSSHKey() *schema.Resource {
	return &schema.Resource{
		CreateContext: resourceDigitalOceanSSHKeyCreate,
		ReadContext:   resourceDigitalOceanSSHKeyRead,
		UpdateContext: resourceDigitalOceanSSHKeyUpdate,
		DeleteContext: resourceDigitalOceanSSHKeyDelete,
		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Schema: map[string]*schema.Schema{
			"name": {
				Type:         schema.TypeString,
				Required:     true,
				ValidateFunc: validation.NoZeroValues,
			},

			"public_key": {
				Type:             schema.TypeString,
				Required:         true,
				ForceNew:         true,
				DiffSuppressFunc: resourceDigitalOceanSSHKeyPublicKeyDiffSuppress,
				ValidateFunc:     validation.NoZeroValues,
			},

			"fingerprint": {
				Type:     schema.TypeString,
				Computed: true,
			},
		},
	}
}

func resourceDigitalOceanSSHKeyPublicKeyDiffSuppress(k, old, new string, d *schema.ResourceData) bool {
	return strings.TrimSpace(old) == strings.TrimSpace(new)
}

func resourceDigitalOceanSSHKeyCreate(ctx context.Context, d *schema.ResourceData, meta interface{}) diag.Diagnostics {
	client := meta.(*CombinedConfig).godoClient()

	// Build up our creation options
	opts := &godo.KeyCreateRequest{
		Name:      d.Get("name").(string),
		PublicKey: d.Get("public_key").(string),
	}

	log.Printf("[DEBUG] SSH Key create configuration: %#v", opts)
	key, _, err := client.Keys.Create(context.Background(), opts)
	if err != nil {
		return diag.Errorf("Error creating SSH Key: %s", err)
	}

	d.SetId(key.ID)
	log.Printf("[INFO] SSH Key: %s", key.ID)

	return resourceDigitalOceanSSHKeyRead(ctx, d, meta)
}

func resourceDigitalOceanSSHKeyRead(ctx context.Context, d *schema.ResourceData, meta interface{}) diag.Diagnostics {
	client := meta.(*CombinedConfig).godoClient()

	id := d.Id()

	key, resp, err := client.Keys.GetByID(context.Background(), id)
	if err != nil {
		// If the key is somehow already destroyed, mark as
		// successfully gone
		if resp != nil && resp.StatusCode == 404 {
			d.SetId("")
			return nil
		}

		return diag.Errorf("Error retrieving SSH key: %s", err)
	}

	d.Set("name", key.Name)
	d.Set("fingerprint", key.Fingerprint)
	d.Set("public_key", key.PublicKey)

	return nil
}

func resourceDigitalOceanSSHKeyUpdate(ctx context.Context, d *schema.ResourceData, meta interface{}) diag.Diagnostics {
	client := meta.(*CombinedConfig).godoClient()

	id := d.Id()

	var newName string
	if v, ok := d.GetOk("name"); ok {
		newName = v.(string)
	}

	log.Printf("[DEBUG] SSH key update name: %#v", newName)
	opts := &godo.KeyUpdateRequest{
		Name: newName,
	}
	_, _, err := client.Keys.UpdateByID(context.Background(), id, opts)
	if err != nil {
		return diag.Errorf("Failed to update SSH key: %s", err)
	}

	return resourceDigitalOceanSSHKeyRead(ctx, d, meta)
}

func resourceDigitalOceanSSHKeyDelete(ctx context.Context, d *schema.ResourceData, meta interface{}) diag.Diagnostics {
	client := meta.(*CombinedConfig).godoClient()

	id := d.Id()

	log.Printf("[INFO] Deleting SSH key: %s", id)
	_, err := client.Keys.DeleteByID(context.Background(), id)
	if err != nil {
		return diag.Errorf("Error deleting SSH key: %s", err)
	}

	d.SetId("")
	return nil
}
